require 'httparty'
require 'multi_xml'

module Mobile
  def self.configure(&block)
    Mobile::Client.new(&block)
  end

  class Client
    include HTTParty
    base_uri 'http://services.mobile.de'

    attr_writer :username, :password, :api_version
    attr_accessor :username, :password, :api_version

    def initialize(&block)
      yield(self)
      self.api_version = '1.0.0' if self.api_version.nil?
      self
    end


    def ads(options={})
      request('get', '/ad/search', options)['result']
    end

    def ad(id, options={})
      request('get', "/ad/#{id}", options)['ad']
    end

    def configured?
      raise Mobile::MissingConfig, "Either username or password is not set in configuration block. Please fix this." unless !username.nil? && !password.nil?
      !username.nil? && !password.nil?
    end

    private

    def request(method, path, opts={})
      opts.merge!({ headers: headers, basic_auth: auth })
      response = self.class.get("/#{api_version}#{path}", opts)
      parsed_body = MultiXml.parse(response.body)
      raise Mobile::BadRequest, "Your search query can not be processed. Unfortunately, we can't provide more details but at least we don't reply with an internal server error any more. Maybe it helps to specify a search parameter which is not boolean or numeric. We will improve this in the future." if parsed_body.nil?
      parsed_body
    end

    def headers
      { "Accept-Language" => "de" }
    end

    def auth
      { username: self.username, password: self.password }
    end
  end
end
